@@ -31,6 +31,9 @@ struct PagePrivate
 
     /* Scanned image data */
     GdkPixbuf *image;
+  
+    /* TRUE if have some page data */
+    gboolean has_data;
 
     /* Expected next scan row */
     gint scan_line;
@@ -86,6 +89,7 @@ page_start (Page *page)
     g_return_if_fail (page != NULL);
 
     page->priv->scan_line = 0;
+    g_signal_emit (page, signals[IMAGE_CHANGED], 0);
 }
 
 
@@ -120,6 +124,14 @@ get_sample (guchar *data, gint depth, gint index)
 }
 
 
+gboolean page_has_data (Page *page)
+{
+    g_return_val_if_fail (page != NULL, FALSE);
+
+    return page->priv->has_data;
+}
+
+
 gint page_get_scan_line (Page *page)
 {
     g_return_val_if_fail (page != NULL, -1);
@@ -204,7 +216,8 @@ page_parse_scan_line (Page *page, ScanLine *line)
         }
         break;
     }
-    
+
+    page->priv->has_data = TRUE;
     page->priv->scan_line = line->number;
     g_signal_emit (page, signals[IMAGE_CHANGED], 0);
 }
@@ -234,10 +247,10 @@ page_finish (Page *page)
 
         g_object_unref (page->priv->image);
         page->priv->image = image;
-
-        g_signal_emit (page, signals[IMAGE_CHANGED], 0);
     }
     page->priv->scan_line = -1;
+
+    g_signal_emit (page, signals[IMAGE_CHANGED], 0);
 }
 
 
@@ -253,10 +266,44 @@ page_get_orientation (Page *page)
 void
 page_set_orientation (Page *page, Orientation orientation)
 {
+    gint left_steps, t;
+
     g_return_if_fail (page != NULL);
 
     if (page->priv->orientation == orientation)
         return;
+  
+    /* Rotate crop */
+    if (page->priv->has_crop) {
+        left_steps = orientation - page->priv->orientation;
+        if (left_steps < 0)
+            left_steps += 4;
+        switch (left_steps) {
+        /* 90 degrees counter-clockwise */
+        case 1:
+            t = page->priv->crop_x;
+            page->priv->crop_x = page->priv->crop_y;
+            page->priv->crop_y = page_get_width (page) - (t + page->priv->crop_width);
+            t = page->priv->crop_width;
+            page->priv->crop_width = page->priv->crop_height;
+            page->priv->crop_height = t;
+            break;
+        /* 180 degrees */
+        case 2:
+            page->priv->crop_x = page_get_width (page) - (page->priv->crop_x + page->priv->crop_width);
+            page->priv->crop_y = page_get_width (page) - (page->priv->crop_y + page->priv->crop_height);
+            break;
+        /* 90 degrees clockwise */
+        case 3:
+            t = page->priv->crop_y;
+            page->priv->crop_y = page->priv->crop_x;
+            page->priv->crop_x = page_get_height (page) - (t + page->priv->crop_height);
+            t = page->priv->crop_width;
+            page->priv->crop_width = page->priv->crop_height;
+            page->priv->crop_height = t;
+            break;
+        }
+    }
 
     page->priv->orientation = orientation;
     g_signal_emit (page, signals[ORIENTATION_CHANGED], 0);
@@ -333,6 +380,24 @@ page_get_height (Page *page)
 }
 
 
+gint
+page_get_scan_width (Page *page)
+{
+    g_return_val_if_fail (page != NULL, 0);
+
+    return gdk_pixbuf_get_width (page->priv->image);
+}
+
+
+gint
+page_get_scan_height (Page *page)
+{
+    g_return_val_if_fail (page != NULL, 0);
+
+    return gdk_pixbuf_get_height (page->priv->image);
+}
+
+
 void
 page_set_no_crop (Page *page)
 {
@@ -348,7 +413,7 @@ page_set_no_crop (Page *page)
 void
 page_set_custom_crop (Page *page, gint width, gint height)
 {
-    gint pw, ph;
+    //gint pw, ph;
 
     g_return_if_fail (page != NULL);
     g_return_if_fail (width >= 1);
@@ -366,7 +431,7 @@ page_set_custom_crop (Page *page, gint width, gint height)
     page->priv->crop_width = width;
     page->priv->crop_height = height;
 
-    pw = page_get_width (page);
+    /*pw = page_get_width (page);
     ph = page_get_height (page);
     if (page->priv->crop_width < pw)
         page->priv->crop_x = (pw - page->priv->crop_width) / 2;
@@ -375,7 +440,7 @@ page_set_custom_crop (Page *page, gint width, gint height)
     if (page->priv->crop_height < ph)
         page->priv->crop_y = (ph - page->priv->crop_height) / 2;
     else
-        page->priv->crop_y = 0;
+        page->priv->crop_y = 0;*/
     
     g_signal_emit (page, signals[CROP_CHANGED], 0);
 }
@@ -396,7 +461,6 @@ page_set_named_crop (Page *page, const gchar *name)
         {"letter", 8.5, 11},
         {"legal", 8.5, 14},
         {"4x6", 4, 6},
-        {"custom", 0, 0},
         {NULL, 0, 0}
     };
     gint i;
@@ -430,14 +494,8 @@ page_set_named_crop (Page *page, const gchar *name)
     }
 
     /* Custom crop, make slightly smaller than original */
-    if (width == 0 || height == 0) {
-        page->priv->crop_width = (int) (pw * 0.8 + 0.5);
-        page->priv->crop_height = (int) (ph * 0.8 + 0.5);
-    }
-    else {
-        page->priv->crop_width = (int) (width * page->priv->dpi + 0.5);
-        page->priv->crop_height = (int) (height * page->priv->dpi + 0.5);
-    }
+    page->priv->crop_width = (int) (width * page->priv->dpi + 0.5);
+    page->priv->crop_height = (int) (height * page->priv->dpi + 0.5);
         
     if (page->priv->crop_width < pw)
         page->priv->crop_x = (pw - page->priv->crop_width) / 2;
@@ -471,10 +529,35 @@ page_rotate_crop (Page *page)
     gint t;
     
     g_return_if_fail (page != NULL);
+  
+    if (!page->priv->has_crop)
+        return;
 
     t = page->priv->crop_width;
     page->priv->crop_width = page->priv->crop_height;
     page->priv->crop_height = t;
+  
+    /* Clip custom crops */
+    if (!page->priv->crop_name) {
+        gint w, h;
+
+        w = page_get_width (page);
+        h = page_get_height (page);
+        
+        if (page->priv->crop_x + page->priv->crop_width > w)
+            page->priv->crop_x = w - page->priv->crop_width;
+        if (page->priv->crop_x < 0) {
+            page->priv->crop_x = 0;
+            page->priv->crop_width = w;
+        }
+        if (page->priv->crop_y + page->priv->crop_height > h)
+            page->priv->crop_y = h - page->priv->crop_height;
+        if (page->priv->crop_y < 0) {
+            page->priv->crop_y = 0;
+            page->priv->crop_height = h;
+        }
+    }
+
     g_signal_emit (page, signals[CROP_CHANGED], 0);
 }
 
